import  java.io.*;
import  java.util.*;
import  java.net.*;
import  java.lang.*;

/*
@version        1.0    
@date	        May 07 1997
@author		Jitong Wang
*/

public class sendmail
{
	private static String filename_recipients;
	private static String filename_email;
	private static Properties properties = new Properties();
	private static String iniFilename = "sendmail.ini";
	private static int mailport = 25;
	private static String mailserver = "";
	private static String hostname;
	private static String recipient;
	private static String username;
	private static Socket socket;
	private static DataInputStream inStream;
	private static PrintStream outStream;
	private static boolean verbose = false;
	private static String email_text = "";

	/**
	The main routine for the sendmail application.
	Determines all properties from the file <tt>sendmail.ini</tt>,
	reads the e-mail and the recipients from text files and
	sends the mail to every recipient.
						*/

	public static void main (String args[]) 
	{
		// Command line processing
									
		if ( args.length > 2 || args.length == 0 )
		{
System.out.println("sendmail application, (c) 1997 Jitong Wang");
System.out.println("http://osprey7.napc.syr.edu/cps616spring97-docs/cb97jwa");
System.out.println("Usage: java sendmail mail-file [recipients-file]");
			System.exit(0);
		}
									
		if ( args.length == 2)
			filename_recipients = args[1];
							        
		filename_email = args[0];
									
									
		// load sendmail-properties-file
		properties = new Properties();
		try {
			properties.load(new FileInputStream(iniFilename));
		} catch ( IOException ioe )
		{
			System.err.println("IO-Error reading file: sendmail.ini");
		  	System.exit(0);
		}
							
		// check, if required properties are set.
		if ( (mailserver = properties.getProperty("mailserver")) == null)
		{
		  	System.err.println("no mailserver specified in file: sendmail.ini");
			System.exit(0);
		}	
				
		if ( (username = properties.getProperty("username")) == null)
		{
		  	System.err.println("no username specified in file: sendmail.ini");
			System.exit(0);
		}
				        
		// check optional properties.	
		String temp;
		try {
			mailport = (new Integer((((temp = properties.getProperty("mailport")) == null) ? "25" : temp))).intValue();
		} catch (NumberFormatException nfe)
		{
			System.out.println("Wrong format of property <mailport>, using default mailport=25");
		}
							
		if (properties.getProperty("verbose") != null )
			verbose = true;
						 			
		// if no recipient-file is specified, then use property <recipient> of file sendmail.ini
		if ( (filename_recipients == null) &&  ((recipient = properties.getProperty("recipient")) == null))
		{
			System.err.println("Error: no recipient specified.");
			System.exit(0);
		}
									
		// Load mail-text
		try {
			DataInputStream dis = new DataInputStream(new FileInputStream(filename_email));
			String line = "";
			while ( (line = dis.readLine())	!= null)
				email_text += line + "\n";
		}
		catch (FileNotFoundException fnfe)
		{
			System.err.println("The file <" + filename_email + "> wasn't found");
		}
		catch ( IOException ioe)
		{
			System.err.println("IO-Error while reading file " + filename_email);
		}
				
				
		// determine local hostname
		try {
			InetAddress ipaddress = InetAddress.getLocalHost();
			if ( (hostname = ipaddress.getHostName()) == null )
				hostname = "localhost";
		} catch (UnknownHostException uhe)
		{
			hostname = "localhost";
		}
						
		// Every line in filename_email is an e-mail-address.
		if ( filename_recipients == null )
		   	mail(recipient, email_text);
		else
		{
			try {
				DataInputStream dis = new DataInputStream(new FileInputStream(filename_recipients));
				while ( (recipient = dis.readLine()) != null)
					mail(recipient, email_text); 
			} catch (FileNotFoundException fnfe)
			{
				System.err.println("The file <" + filename_recipients + "> wasn't found");
			}
			catch ( IOException ioe )
			{
				System.err.println("IO-Error while reading file " + filename_recipients);
			}
		}
	}

	private static void mail (String recipient, String email_text) 
	{
		String reply;
		
		try
		{
			socket = new Socket(mailserver, mailport);	
			inStream = new DataInputStream( socket.getInputStream() );
			outStream = new PrintStream(socket.getOutputStream() );
		
			reply = inStream.readLine();
			debug_print(reply);
		
			doCommand("HELO " + hostname, true );
			doCommand("MAIL FROM: " + username, true );
			doCommand("RCPT TO: " + recipient, true );
			doCommand("DATA", true);
		
			doCommand( email_text, false);
			doCommand( "\n.\n", false);
			socket.close();
			System.out.println("mail sent: " + recipient);
		}
		catch(IOException ioe)
		{
			System.err.println("Socket-IO-Error: " + ioe );
		}
	}

	private static void doCommand (String command, boolean ExpectingResponse) 
	{
		// note that this method should be checking the status of the
		// command and throwing an exception if an error occurs in the
		// SMTP conversation.  Perhaps next time...
		
		String reply = new String();
		
		try
		{
		   outStream.println( command );
		   outStream.flush();
		   debug_print( command );
		   if ( ExpectingResponse )
		   {
		      debug_print(inStream.readLine());
		   }
		
		}
		catch(IOException ioe)
		{
		   outStream.println( "\n.\n" );
		   debug_print("Socket-IO-Error: " + ioe );
		}
		return;
	}

	private static void debug_print (String message) 
	{
		if ( verbose )
		{
		   System.out.println( message ) ;
		}
		return;
	}

	private sendmail () 
	{
		
	}

}

